/* $Id: info.h,v 1.11 2005/04/05 12:02:56 ellson Exp $ $Revision: 1.11 $ */
/* vim:set shiftwidth=4 ts=8: */
/*
 This software is part of the graphviz package
 Copyright (c) 1995-2004 AT&T Corp.
 Licensed under the Common Public License, Version 1.0 (the "License")
 Any use, reproduction or distribution of this software constitutes
 acceptance of the License.  A copy of the License is available at:
     http://www.research.att.com/sw/license/cpl-1.0.html
 (with md5 checksum 201a9e4ba08a96f5d9677315d8ce1463)
*/

#ifndef INFO_H
#define INFO_H

#include "common/Dynagraph.h"
#include "common/freelist.h"
#include "voronoi/site.h"

namespace Voronoi {

struct PtItem {           /* Point std::list */
	PtItem*    next;
	Coord             p;
	PtItem() : next(0) {}
};
struct Info {                  /* Info concerning site */
	Layout::Node *layoutN;     /* libgraph node */
	Site site;                 /* site used by voronoi code */
	bool overlaps;             /* true if node overlaps other nodes */
	PtItem *verts;             /* sorted std::list of vertices of */
							   /* voronoi polygon */
	Info() : layoutN(0),overlaps(false),verts(0) {}
};
struct Infos {
	Freelist<PtItem> fpoints;


	std::vector<Info> nodes;			/* Array of node info */

	Infos(int N) : fpoints(ROUND(sqrt((double)N))),nodes(N) {}

	/* Insert vertex into sorted std::list */
	void addVertex (Site*, Coord);  
};
inline double dist(Site *s, Site *t) {
	return ::dist(s->coord,t->coord);
}

} // namespace Voronoi
#endif // INFO_H
